@@ -130,6 +130,7 @@ proc finder_next {{direction variable}} {
     global finder_direction
     global finder_pattern
     global finder_current_index
+    global tcl_jp
 
     if {$finder_pattern == ""} {
 	output_information [_ "The pattern is empty."]
@@ -162,8 +163,15 @@ proc finder_next {{direction variable}} {
         output_information [format [_ "Not found \`%s\'."] $finder_pattern]
         return
     }
-    .text.text tag add sel $index \
-	    "$index + [kstring length $finder_pattern] chars"
+
+    # use kstring in case Tcl8.0.x_jp
+    if {$tcl_jp} {
+	.text.text tag add sel $index \
+		"$index + [kstring length $finder_pattern] chars"
+    } else {
+	.text.text tag add sel $index \
+		"$index + [string length $finder_pattern] chars"
+    }
     .text.text see $index
 
     if {$direction == "forward"} {
