@@ -53,6 +53,9 @@ set ndtp_current_book 0
 # current bitmap size
 set ndtp_current_size 0
 
+# current text position
+set ndtp_current_text_position ""
+
 # created bitmap size
 set ndtp_cache_size 0
 
@@ -83,6 +86,7 @@ proc ndtp_initialize {} {
     global ndtp_books
     global ndtp_current_book
     global ndtp_current_size
+    global ndtp_current_text_position
     global ndtp_cache_size
     global ndtp_titles
     global ndtp_pages_menu
@@ -137,6 +141,7 @@ proc ndtp_initialize {} {
 
     set ndtp_current_book 0
     set ndtp_current_size 0
+    set ndtp_current_text_position ""
 }
 
 #
@@ -173,6 +178,9 @@ proc ndtp_open {hostname port books} {
 	return 0
     }
     fconfigure $ndtp_socket -translation {lf lf}
+
+    # next command is supported by tcl 8.1 and higher
+    catch {fconfigure $ndtp_socket -encoding euc-jp}
     
     #
     # Send the `A' command.
@@ -281,6 +289,7 @@ proc ndtp_select_book {book} {
     global ndtp_books
     global ndtp_current_book
     global ndtp_current_size
+    global ndtp_current_text_position
     global ndtp_pages_menu
     global ndtp_pages_copyright
     global ndtp_pages_word
@@ -331,6 +340,7 @@ proc ndtp_select_book {book} {
     #
     set ndtp_current_book 0
     set ndtp_current_size 0
+    set ndtp_current_text_position ""
     if {![ndtp_put_line "L$book"]} {
 	set ndtp_cache_indexes($book) 0
 	set ndtp_cache_sizes($book) 0
@@ -460,6 +470,7 @@ proc ndtp_select_book {book} {
 proc ndtp_query_word {word positions headings} {
     global ndtp_error_message
     global ndtp_default_method
+    global tcl_jp
     upvar $positions up_positions
     upvar $headings up_headings
 
@@ -477,40 +488,72 @@ proc ndtp_query_word {word positions headings} {
     #
     # Adjust $word according with $ndtp_default_method.
     #
-    if {$ndtp_default_method == "keyword" \
-	    && ![kanji string match {*;*} $word]} {
-	set word "$word;"
-    } elseif {$ndtp_default_method == "endword" \
-	    && ![kanji string match {\**} $word]} {
-	set word "*$word"
-    } elseif {$ndtp_default_method == "word" \
-	    && ![kanji string match {*\*} $word]} {
-	set word "$word*"
-    }
 
-    #
-    # Set a request line.
-    #
-    if {[kanji string match {*;*} $word]} {
-	regsub -all {[ 	]*;[ 	]*} $word {	} word
-	set line "Pj$word"
-    } elseif {[kanji string match {\**} $word]} {
-	set length [kstring length $word]
-	set reversed_word ""
-	for {set i 0} {$i < $length} {incr i} {
-	    set c [kstring index $word $i]
-	    set reversed_word "$c$reversed_word"
+    # 2003 Masaki Shinomiya (supporting Tcl/Tk8.1 and higher)
+    # use kanji string match in case Tcl8.0.x_jp
+    if {$tcl_jp} {
+	if {$ndtp_default_method == "keyword" \
+		    && ![kanji string match {*;*} $word]} {
+		set word "$word;"
+	} elseif {$ndtp_default_method == "endword" \
+		    && ![kanji string match {\**} $word]} {
+		set word "*$word"
+	} elseif {$ndtp_default_method == "word" \
+		    && ![kanji string match {*\*} $word]} {
+		set word "$word*"
 	}
-	if {[kanji code $word] == {ANY}} {
-	    set line "PA$reversed_word"
+	#
+	# Set a request line.
+	#
+	if {[kanji string match {*;*} $word]} {
+		regsub -all {[ 	]*;[ 	]*} $word {	} word
+		set line "Pj$word"
+	} elseif {[kanji string match {\**} $word]} {
+		set length [kstring length $word]
+		set reversed_word ""
+		for {set i 0} {$i < $length} {incr i} {
+		    set c [kstring index $word $i]
+		    set reversed_word "$c$reversed_word"
+		}
+		if {[kanji code $word] == {ANY}} {
+		    set line "PA$reversed_word"
+		} else {
+		    set line "PK$reversed_word"
+		}
 	} else {
-	    set line "PK$reversed_word"
+		if {[kanji code $word] == {ANY}} {
+		    set line "Pa$word"
+		} else {
+		    set line "Pk$word"
+		}
 	}
     } else {
-	if {[kanji code $word] == {ANY}} {
-	    set line "Pa$word"
+	if {$ndtp_default_method == "keyword" \
+			&& ![string match {*;*} $word]} {
+		set word "$word;"
+	} elseif {$ndtp_default_method == "endword" \
+			&& ![string match {\**} $word]} {
+		set word "*$word"
+	} elseif {$ndtp_default_method == "word" \
+			&& ![string match {*\*} $word]} {
+		set word "$word*"
+	}
+	#
+	# Set a request line.
+	#
+	if {[string match {*;*} $word]} {
+		regsub -all {[ 	]*;[ 	]*} $word {	} word
+		set line "Pj$word"
+	} elseif {[string match {\**} $word]} {
+		set length [string length $word]
+		set reversed_word ""
+		for {set i 0} {$i < $length} {incr i} {
+		    set c [string index $word $i]
+		    set reversed_word "$c$reversed_word"
+		}
+		    set line "PA$reversed_word"
 	} else {
-	    set line "Pk$word"
+		    set line "Pa$word"
 	}
     }
 
@@ -555,9 +598,10 @@ proc ndtp_get_text {position text} {
     global ndtp_current_book
     global ndtp_pages_menu
     global ndtp_pages_copyright
+    global ndtp_current_text_position
     upvar $text up_text
 
-    set text {}
+    set up_text {}
 
     #
     # Get a real position for menu and copyright request.
@@ -577,6 +621,16 @@ proc ndtp_get_text {position text} {
 	    }
 	    set position "$ndtp_pages_copyright($ndtp_current_book):0"
 	}
+	"forward" {
+	    set position [ndtp_get_forward_position]
+	}
+	"backward" {
+	    set position [ndtp_get_backward_position]
+	}
+    }
+
+    if {$position == ""} {
+	return 0
     }
 
     #
@@ -607,9 +661,70 @@ proc ndtp_get_text {position text} {
 	append up_text $line "\n"
     }
 
+    set ndtp_current_text_position $position
+
     return 1
 }
 
+#
+# Get text position of next paragraph.
+#
+proc ndtp_get_forward_position {} {
+    global ndtp_error_message
+    global ndtp_current_book
+    global ndtp_pages_menu
+    global ndtp_pages_copyright
+    global ndtp_current_text_position
+
+    #
+    # Send the `XS' command.
+    #
+    if {$ndtp_current_text_position == ""} {
+	return ""
+    }
+    if {![ndtp_put_line "XS+$ndtp_current_text_position"]} {
+	return $ndtp_current_text_position
+    }
+    if {![ndtp_get_line line]} {
+	return $ndtp_current_text_position
+    }
+    if {![string match {\$XS*} $line]} {
+	return $ndtp_current_text_position
+    }
+    regsub {^\$XS} $line "" line
+
+    return $line
+}
+
+#
+# Get text position of previous paragraph.
+#
+proc ndtp_get_backward_position {} {
+    global ndtp_error_message
+    global ndtp_current_book
+    global ndtp_pages_menu
+    global ndtp_pages_copyright
+    global ndtp_current_text_position
+
+    #
+    # Send the `XS' command.
+    #
+    if {$ndtp_current_text_position == ""} {
+	return ""
+    }
+    if {![ndtp_put_line "XS-$ndtp_current_text_position"]} {
+	return $ndtp_current_text_position
+    }
+    if {![ndtp_get_line line]} {
+	return $ndtp_current_text_position
+    }
+    if {![string match {\$XS*} $line]} {
+	return $ndtp_current_text_position
+    }
+    regsub {^\$XS} $line "" line
+    return $line
+}
+
 #
 # Select a bitmap size, and get all bitmap data.
 #
@@ -625,7 +740,7 @@ proc ndtp_select_bitmap_size {size {load_mode "pre-load"}} {
     global ndtp_cache_all_bitmaps
 
     #
-    # If $book has been selected, return immediately.
+    # If $size has been selected, return immediately.
     #
     if {$size == $ndtp_current_size} {
 	return 1
@@ -892,6 +1007,7 @@ proc ndtp_have_copyright {book} {
 #
 proc ndtp_get_line {line} {
     global tcl_platform
+    global tcl_jp
     global ndtp_error_message
     global ndtp_socket
     upvar $line up_line
@@ -905,7 +1021,8 @@ proc ndtp_get_line {line} {
 	return 0
     }
 
-    if {$tcl_platform(platform) != "unix"} {
+    # use kanji conversion in case Tcl8.0.x_jp
+    if {$tcl_platform(platform) != "unix" && $tcl_jp} {
 	set up_line [kanji conversion EUC SJIS $up_line]
     }
 
@@ -917,10 +1034,12 @@ proc ndtp_get_line {line} {
 #
 proc ndtp_put_line {line} {
     global tcl_platform
+    global tcl_jp
     global ndtp_error_message
     global ndtp_socket
 
-    if {$tcl_platform(platform) != "unix"} {
+    # use kanji conversion in case Tcl8.0.x_jp
+    if {$tcl_platform(platform) != "unix" && $tcl_jp} {
 	set line [kanji conversion SJIS EUC $line]
     }
 
